@@ -182,65 +182,40 @@ ngx_http_groonga_logger_log(grn_ctx *ctx, grn_log_level level,
                             void *user_data)
 {
   ngx_open_file_t *file = user_data;
-  const char level_marks[] = " EACewnid-";
+  char level_marks[] = " EACewnid-";
   u_char buffer[NGX_MAX_ERROR_STR];
-  u_char *last;
-  size_t prefix_size;
-  size_t message_size;
-  size_t location_size;
-  size_t postfix_size;
-  size_t log_message_size;
 
   if (!file) {
     return;
   }
 
-#define LOG_PREFIX_FORMAT "%s|%c|%s"
-  prefix_size =
-    strlen(timestamp) +
-    1 /* | */ +
-    1 /* %c */ +
-    1 /* | */ +
-    strlen(title) +
-    1 /* a space */;
-  message_size = strlen(message);
+  ngx_http_groonga_write_fd(file->fd,
+                            buffer, NGX_MAX_ERROR_STR,
+                            timestamp, strlen(timestamp));
+  ngx_write_fd(file->fd, "|", 1);
+  ngx_write_fd(file->fd, level_marks + level, 1);
+  ngx_write_fd(file->fd, "|", 1);
   if (location && *location) {
-    location_size = 1 /* a space */ + strlen(location);
-  } else {
-    location_size = 0;
-  }
-  postfix_size = 1 /* \n */;
-  log_message_size = prefix_size + message_size + location_size + postfix_size;
-
-  if (log_message_size > NGX_MAX_ERROR_STR) {
-    last = ngx_slprintf(buffer, buffer + NGX_MAX_ERROR_STR,
-                        LOG_PREFIX_FORMAT,
-                        timestamp, *(level_marks + level), title);
-    ngx_write_fd(file->fd, buffer, last - buffer);
     ngx_http_groonga_write_fd(file->fd,
                               buffer, NGX_MAX_ERROR_STR,
-                              message, message_size);
-    if (location_size > 0) {
-      ngx_write_fd(file->fd, " ", 1);
+                              location, strlen(location));
+    ngx_write_fd(file->fd, ": ", 2);
+    if (title && *title) {
       ngx_http_groonga_write_fd(file->fd,
                                 buffer, NGX_MAX_ERROR_STR,
-                                location, location_size);
+                                title, strlen(title));
+      ngx_write_fd(file->fd, " ", 1);
     }
-    ngx_write_fd(file->fd, "\n", 1);
   } else {
-    if (location && *location) {
-      last = ngx_slprintf(buffer, buffer + NGX_MAX_ERROR_STR,
-                          LOG_PREFIX_FORMAT " %s %s\n",
-                          timestamp, *(level_marks + level), title, message,
-                          location);
-    } else {
-      last = ngx_slprintf(buffer, buffer + NGX_MAX_ERROR_STR,
-                          LOG_PREFIX_FORMAT " %s\n",
-                          timestamp, *(level_marks + level), title, message);
-    }
-    ngx_write_fd(file->fd, buffer, last - buffer);
+    ngx_http_groonga_write_fd(file->fd,
+                              buffer, NGX_MAX_ERROR_STR,
+                              title, strlen(title));
+    ngx_write_fd(file->fd, " ", 1);
   }
-#undef LOG_PREFIX_FORMAT
+  ngx_http_groonga_write_fd(file->fd,
+                            buffer, NGX_MAX_ERROR_STR,
+                            message, strlen(message));
+  ngx_write_fd(file->fd, "\n", 1);
 }
 
 static void
@@ -258,7 +233,7 @@ ngx_http_groonga_logger_fin(grn_ctx *ctx, void *user_data)
 
 static grn_logger ngx_http_groonga_logger = {
   GRN_LOG_DEFAULT_LEVEL,
-  GRN_LOG_TIME | GRN_LOG_MESSAGE,
+  GRN_LOG_TIME | GRN_LOG_MESSAGE | GRN_LOG_PID,
   NULL,
   ngx_http_groonga_logger_log,
   ngx_http_groonga_logger_reopen,
@@ -691,15 +666,15 @@ ngx_http_groonga_handler_create_data(ngx_http_request_t *r,
 static void
 ngx_http_groonga_handler_process_command_path(ngx_http_request_t *r,
                                               ngx_str_t *command_path,
-                                              ngx_http_groonga_handler_data_t *data)
+                                              ngx_http_groonga_handler_data_t *data,
+                                              int flags)
 {
   grn_obj uri;
 
   GRN_TEXT_INIT(&uri, 0);
   GRN_TEXT_PUTS(context, &uri, "/d/");
   GRN_TEXT_PUT(context, &uri, command_path->data, command_path->len);
-  grn_ctx_send(context, GRN_TEXT_VALUE(&uri), GRN_TEXT_LEN(&uri),
-               GRN_NO_FLAGS);
+  grn_ctx_send(context, GRN_TEXT_VALUE(&uri), GRN_TEXT_LEN(&uri), flags);
   data->rc = context->rc;
   ngx_http_groonga_context_log_error(r->connection->log);
   GRN_OBJ_FIN(context, &uri);
@@ -800,13 +775,19 @@ ngx_http_groonga_send_body(ngx_http_request_t *r,
              line_current < line_end;
              line_current++) {
           size_t line_length;
+          int flags = GRN_NO_FLAGS;
 
           if (*line_current != '\n') {
             continue;
           }
 
           line_length = line_current - line_start + 1;
-          grn_ctx_send(context, line_start, line_length, GRN_NO_FLAGS);
+          if (line_current + 1 == line_end &&
+              !chain->next &&
+              rest_buffer_size == 0) {
+            flags |= GRN_CTX_TAIL;
+          }
+          grn_ctx_send(context, line_start, line_length, flags);
           line_start_offset += line_length;
           line_start += line_length;
           ngx_http_groonga_context_log_error(log);
@@ -840,7 +821,7 @@ ngx_http_groonga_send_body(ngx_http_request_t *r,
     grn_ctx_send(context,
                  GRN_TEXT_VALUE(&line_buffer),
                  GRN_TEXT_LEN(&line_buffer),
-                 GRN_NO_FLAGS);
+                 GRN_CTX_TAIL);
     ngx_http_groonga_context_log_error(log);
     if (context->rc != GRN_SUCCESS && data->rc == GRN_SUCCESS) {
       data->rc = context->rc;
@@ -878,7 +859,10 @@ ngx_http_groonga_handler_process_load(ngx_http_request_t *r,
     return;
   }
 
-  ngx_http_groonga_handler_process_command_path(r, command_path, data);
+  ngx_http_groonga_handler_process_command_path(r,
+                                                command_path,
+                                                data,
+                                                GRN_NO_FLAGS);
   if (data->rc != GRN_SUCCESS) {
     return;
   }
@@ -926,7 +910,14 @@ ngx_http_groonga_handler_send_response(ngx_http_request_t *r,
 
   /* set the 'Content-type' header */
   if (r->headers_out.content_type.len == 0) {
-    content_type = grn_ctx_get_mime_type(context);
+    grn_obj *foot = &(data->typed.foot);
+    if (grn_ctx_get_output_type(context) == GRN_CONTENT_JSON &&
+        GRN_TEXT_LEN(foot) > 0 &&
+        GRN_TEXT_VALUE(foot)[GRN_TEXT_LEN(foot) - 1] == ';') {
+      content_type = "application/javascript";
+    } else {
+      content_type = grn_ctx_get_mime_type(context);
+    }
     ngx_http_groonga_handler_set_content_type(r, content_type);
   }
 
@@ -993,7 +984,10 @@ ngx_http_groonga_handler_get(ngx_http_request_t *r)
     return rc;
   }
 
-  ngx_http_groonga_handler_process_command_path(r, &command_path, data);
+  ngx_http_groonga_handler_process_command_path(r,
+                                                &command_path,
+                                                data,
+                                                GRN_CTX_TAIL);
 
   /* discard request body, since we don't need it here */
   rc = ngx_http_discard_request_body(r);
@@ -1213,6 +1207,11 @@ ngx_http_groonga_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
 {
   ngx_http_groonga_loc_conf_t *prev = parent;
   ngx_http_groonga_loc_conf_t *conf = child;
+  ngx_flag_t enabled = 0;
+
+  if (conf->enabled != NGX_CONF_UNSET) {
+    enabled = conf->enabled;
+  }
 
   ngx_conf_merge_str_value(conf->database_path, prev->database_path, NULL);
   ngx_conf_merge_value(conf->database_auto_create,
@@ -1226,7 +1225,7 @@ ngx_http_groonga_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
                            NGX_HTTP_GROONGA_LOG_PATH);
   if (!conf->log_file &&
       ngx_str_is_custom_path(&(conf->log_path)) &&
-      conf->enabled) {
+      enabled) {
     conf->log_file = ngx_conf_open_file(cf->cycle, &(conf->log_path));
     if (!conf->log_file) {
       ngx_log_error(NGX_LOG_ERR, cf->cycle->log, 0,
@@ -1242,7 +1241,7 @@ ngx_http_groonga_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
                            NGX_HTTP_GROONGA_QUERY_LOG_PATH);
   if (!conf->query_log_file &&
       ngx_str_is_custom_path(&(conf->query_log_path)) &&
-      conf->enabled) {
+      enabled) {
     conf->query_log_file = ngx_conf_open_file(cf->cycle,
                                               &(conf->query_log_path));
     if (!conf->query_log_file) {
@@ -1305,6 +1304,41 @@ ngx_http_groonga_each_loc_conf(ngx_http_conf_ctx_t *http_conf,
     ngx_http_groonga_each_loc_conf_in_tree(location_conf->static_locations,
                                            callback,
                                            user_data);
+
+#if NGX_PCRE
+    if (location_conf->regex_locations) {
+      ngx_uint_t j;
+      for (j = 0; location_conf->regex_locations[j]; j++) {
+        ngx_http_core_loc_conf_t *regex_location_conf;
+
+        regex_location_conf = location_conf->regex_locations[j];
+        if (regex_location_conf->handler == ngx_http_groonga_handler) {
+          callback(regex_location_conf->loc_conf[ngx_http_groonga_module.ctx_index],
+                   user_data);
+        }
+      }
+    }
+#endif
+  }
+}
+
+static void
+ngx_http_groonga_set_logger_callback(ngx_http_groonga_loc_conf_t *location_conf,
+                                     void *user_data)
+{
+  ngx_http_groonga_database_callback_data_t *data = user_data;
+
+  data->rc = ngx_http_groonga_context_init_logger(location_conf,
+                                                  data->pool,
+                                                  data->log);
+  if (data->rc != NGX_OK) {
+    return;
+  }
+  data->rc = ngx_http_groonga_context_init_query_logger(location_conf,
+                                                        data->pool,
+                                                        data->log);
+  if (data->rc != NGX_OK) {
+    return;
   }
 }
 
@@ -1467,6 +1501,20 @@ ngx_http_groonga_init_process(ngx_cycle_t *cycle)
   grn_default_logger_set_path(NGX_HTTP_GROONGA_LOG_PATH);
 #endif
 
+  http_conf =
+    (ngx_http_conf_ctx_t *)ngx_get_conf(cycle->conf_ctx, ngx_http_module);
+
+  data.log = cycle->log;
+  data.pool = cycle->pool;
+  data.rc = NGX_OK;
+  ngx_http_groonga_each_loc_conf(http_conf,
+                                 ngx_http_groonga_set_logger_callback,
+                                 &data);
+
+  if (data.rc != NGX_OK) {
+    return data.rc;
+  }
+
   rc = grn_init();
   if (rc != GRN_SUCCESS) {
     return NGX_ERROR;
@@ -1479,12 +1527,6 @@ ngx_http_groonga_init_process(ngx_cycle_t *cycle)
     return NGX_ERROR;
   }
 
-  http_conf =
-    (ngx_http_conf_ctx_t *)ngx_get_conf(cycle->conf_ctx, ngx_http_module);
-
-  data.log = cycle->log;
-  data.pool = cycle->pool;
-  data.rc = NGX_OK;
   ngx_http_groonga_each_loc_conf(http_conf,
                                  ngx_http_groonga_open_database_callback,
                                  &data);
